#!/bin/bash

APTCOMMAND=/usr/bin/aptitude
APT_GET=/usr/bin/apt-get
AWK=/usr/bin/awk
CAT=/bin/cat
EGREP=/bin/egrep
ECHO=/bin/echo
GREP=/bin/grep
INFOCLIENT=/linuxcash/cash/bin/InfoClient
RM=/bin/rm
TOUCH=/usr/bin/touch
TR=/usr/bin/tr
TRUE=/bin/true
SLEEP=/bin/sleep

APTARG="-y -d -o DPkg::Options::=--force-confold"
CONFIGFILE=/linuxcash/cash/conf/artix-upgrade.conf
DEFINEFILE=/linuxcash/cash/bin/artix-upgrade.def
DOWNLOAD_LIMIT=25
LOGFILE=/linuxcash/logs/current/artix-upgrade.log
IMMEDIATE=
OPTIONS=
PKG_LIST=/linuxcash/cash/conf/artix-packages.list
RUNSLEEP=240
SUDO=

. $DEFINEFILE

UPGRADE_LIST=/tmp/artix-upgrade.list

if [ -f $CONFIGFILE ] ; then
    . $CONFIGFILE
fi

function tryupgrade() {
    setstatus UPGRADE_DOWNLOAD
    DONWLOAD_COMMAND=$1
    TARGETS=$2
    SUCCESS=1
    TIME=3
    CACHE_DIR=/var/cache/apt/archives

    log_info "Запрос [${DONWLOAD_COMMAND}]"
    for i in 1 2 3 4 5 ; do
        log_info "Попытка $i..."
        if $DONWLOAD_COMMAND ; then
            SUCCESS=0
            for ITEM in $(cat $TARGETS | grep --regexp='^Inst' | sed -e 's/^Inst \([^ ]*\)[^(]*(\([^ ]*\).*$/\1_\2/') ; do
                PACKAGE=$(echo $ITEM | sed -e 's/:/%3a/')
                if ! ls $CACHE_DIR/$PACKAGE* >/dev/null 2>&1 ; then
                    log_info "Отсутствует пакет $PACKAGE"
                    SUCCESS=1
                    break
                fi
            done
        fi
        if [ "$SUCCESS" = "0" ] ; then
            log_info "Выполнено успешно!"
            break
        fi
        log_info "Ошибка, повторная попытка будет выполнена через ${TIME} мин."
        $SLEEP "${TIME}m"
    done
    return $SUCCESS
}

function atexit() {
    RESCODE=$?
    [ "$RESCODE" != "0" ] && setstatus EXECUTE_ERROR
    $SUDO $RM -f $UPGRADE_LIST
    exit $RESCODE
}

trap atexit 0 1 2 5 15

TEMP=`/usr/bin/getopt -o il:r:st:v --long immediate,dl-limit:,result:,sudo,timeout:,verbose -- "$@"`
if [ $? != 0 ] ; then /bin/echo "Terminating..." >&2 ; exit 1 ; fi

# Note the quotes around `$TEMP': they are essential!
eval set -- "$TEMP"
while /bin/true ; do
    case "$1" in
        -i|--immediate) IMMEDIATE="yes" ; shift ;;
        -l|--dl-limit) DOWNLOAD_LIMIT=$2 ; shift 2 ;;
        -r|--result) RESULT=$2 ; shift 2 ;;
        -s|--sudo) SUDO="/usr/bin/sudo -E" ; shift ;;
        -t|--timeout) RUNSLEEP=$2 ; shift 2 ;;
        --) shift ; break ;;
        *) /bin/echo "Internal error!" ; exit 1 ;;
    esac
done

if [ -e $PKG_DOWNLOAD ] ; then 
    [ -e $PKG_INSTALL_LOCK ] && setstatus UPGRADE_AVAILABLE_LOCK || setstatus UPGRADE_AVAILABLE
    exit 0
fi

# Random sleep time before run
if [ -z $IMMEDIATE ] ; then
    RANDOM=$(/bin/dd if=/dev/urandom count=1 2> /dev/null | /usr/bin/cksum | /usr/bin/cut -c"1-5")
    TIME=$(($RANDOM % $RUNSLEEP))

    log_info "Процесс загрузки обновлений отложен на ${TIME} мин."
#    setstatus UPGRADE_WAIT
    $SLEEP "${TIME}m"
fi

log_info "Запуск процесса загрузки обновлений"

[ ! -z $DOWNLOAD_LIMIT ] && OPTIONS="-o Acquire::http::Dl-Limit=$DOWNLOAD_LIMIT"
UPGRADE_AVAILABLE=

log_info "Обновление индексных файлов"
$SUDO $APTCOMMAND update || setstatus INDEX_ERROR EXIT
if [ -e $PKG_INSTALL ] ; then
    log_info "Установка новых пакетов..."
    $SUDO $RM -f $PKG_INSTALL_LIST
    $CAT $PKG_LIST | $EGREP -v '^[[:space:]]*#' | $AWK -F '#' '{print $1}' | $TR -d ' ' > $PKG_INSTALL_LIST
    if [ -f $PKG_VERSION ] ; then
        PACKAGES_TEMP=`$CAT $PKG_INSTALL_LIST`
        $SUDO $RM -f $PKG_INSTALL_LIST
        for NAME in $PACKAGES_TEMP ; do
            $GREP "$NAME=" $PKG_VERSIONLIST >> $PKG_INSTALL_LIST || $ECHO "$NAME" >> $PKG_INSTALL_LIST
        done
    fi
    PACKAGES=`$CAT $PKG_INSTALL_LIST`
    $SUDO $APT_GET --simulate install $PACKAGES 2>&1 >$UPGRADE_LIST || setstatus DOWNLOAD_ERROR EXIT
    if $GREP 'Inst' $UPGRADE_LIST >/dev/null ; then
        tryupgrade "$SUDO $VISUAL $APTCOMMAND $APTARG $OPTIONS install $PACKAGES" $UPGRADE_LIST && UPGRADE_AVAILABLE="YES" || $TRUE
        [ -z $UPGRADE_AVAILABLE ] && setstatus DOWNLOAD_ERROR EXIT
    fi
elif [ ! -e $PKG_UPGRADE_LOCK ] ; then
    log_info "Обновление системы..."
    $SUDO $APT_GET --simulate dist-upgrade 2>&1 >$UPGRADE_LIST || setstatus DOWNLOAD_ERROR EXIT
    if $GREP 'Inst' $UPGRADE_LIST >/dev/null ; then
        tryupgrade "$SUDO $VISUAL $APTCOMMAND $APTARG $OPTIONS full-upgrade" $UPGRADE_LIST && UPGRADE_AVAILABLE="YES" || $TRUE
        [ -z $UPGRADE_AVAILABLE ] && setstatus DOWNLOAD_ERROR EXIT
    fi
else
    setstatus UPGRADE_LOCKED
fi

if [ ! -z $UPGRADE_AVAILABLE ] ; then
    $TOUCH $PKG_DOWNLOAD
    if [ ! -e $PKG_INSTALL_LOCK ] ; then
        setstatus UPGRADE_AVAILABLE
        [ -x $INFOCLIENT ] && $INFOCLIENT --encoding="UTF-8" --set-alert="Доступны обновления" || $TRUE
    else
        setstatus UPGRADE_AVAILABLE_LOCK
    fi
else
    setstatus NOTHING_TODO
fi
